@@ -1,9 +1,9 @@
 # created by Dmitrey
 from numpy import copy, isnan, array, argmax, abs, vstack, zeros, any, isfinite, all, where, asscalar, \
-sign, dot, sqrt, array_equal, nanmax, inf, hstack, isscalar, logical_or, matrix, asfarray, prod, arange, ndarray
-from numpy.linalg import norm
-from nonOptMisc import Copy
-from pointProjection import pointProjection
+dot, sqrt, nanmax, logical_or, matrix, prod, arange, ndarray, asarray, sum
+from ooMisc import norm
+from nonOptMisc import Copy, isPyPy
+#from pointProjection import pointProjection
 empty_arr = array(())
 try:
     from scipy.sparse import isspmatrix, csr_matrix
@@ -22,16 +22,19 @@ class Point:
     def __init__(self, p, x, *args, **kwargs):
         self.p = p
         self.x = copy(x)
+        self.isMultiArray = self.x.ndim > 1
         for i, arg in enumerate(args):
             setattr(self, '_' + self.__expectedArgs__[i], args[i])
-        for name, val in kwargs.iteritems():
+        for name, val in kwargs.items():
             setattr(self, '_' + name, val)
         #assert self.x is not None
 
-    def f(self):
+    def f(self):#, _linePointDescriptor = None):
         if not hasattr(self, '_f'): 
-            if self.p._baseClassName == 'NonLin':
-                self._f = self.p.f(self.x)
+            # TODO: rework this: self.p.probType!='IP'
+            if self.p._baseClassName == 'NonLin' and self.p.probType!='IP':
+                func = self.p.f if self.p.isObjFunValueASingleNumber else self.p.F
+                self._f = func(self.x)#, _linePointDescriptor = _linePointDescriptor)
             else:
                 self._f = self.p.objFunc(self.x)
         return copy(self._f)
@@ -42,9 +45,16 @@ class Point:
         return Copy(self._df)
 
     def c(self, ind=None):
-        if not self.p.userProvided.c: return empty_arr.copy()
+        if not self.p.userProvided.c: 
+            return empty_arr.copy()
         if ind is None:
-            if not hasattr(self, '_c'): self._c = self.p.c(self.x)
+            if not hasattr(self, '_c'): 
+                self._c = self.p.c(self.x)
+                self._nNaNs_C = isnan(self._c).sum(self._c.ndim-1)
+                if self.isMultiArray:
+                    tmp = self._c
+                    delattr(self, '_c')
+                    return tmp
             return copy(self._c)
         else:
             if hasattr(self, '_c'): return copy(self._c[ind])
@@ -57,7 +67,6 @@ class Point:
             if not hasattr(self, '_dc'): self._dc = self.p.dc(self.x)
             return Copy(self._dc)
         else:
-            
             if hasattr(self, '_dc'): return Copy(self._dc[ind])
             else: return Copy(self.p.dc(self.x, ind))
 
@@ -65,7 +74,13 @@ class Point:
     def h(self, ind=None):
         if not self.p.userProvided.h: return empty_arr.copy()
         if ind is None:
-            if not hasattr(self, '_h'): self._h = self.p.h(self.x)
+            if not hasattr(self, '_h'): 
+                self._h = self.p.h(self.x)
+                self._nNaNs_H = isnan(self._h).sum(self._h.ndim-1)
+                if self.isMultiArray:
+                    tmp = self._h
+                    delattr(self, '_h')
+                    return tmp
             return copy(self._h)
         else:
             if hasattr(self, '_h'): return copy(self._h[ind])
@@ -73,7 +88,6 @@ class Point:
 
     def dh(self, ind=None):
         if not self.p.userProvided.h: return empty_arr.copy().reshape(0, self.p.n)
-        #raise 0
         if ind is None:
             if not hasattr(self, '_dh'): self._dh = self.p.dh(self.x)
             return Copy(self._dh)
@@ -111,40 +125,46 @@ class Point:
         if not hasattr(self, '_ub'): self._ub = self.x - self.p.ub
         return copy(self._ub)
 
-    def mr(self, retAll = False):
+    def mr(self, retAll = False, checkBoxBounds = True):
         # returns max residual
+        
         if not hasattr(self, '_mr'):
-            r, fname, ind = 0, None, None
-            ineqs = ['lin_ineq', 'lb', 'ub']
+            r, fname, ind = 0, None, 0
+            if self.isMultiArray: 
+                r = zeros(self.x.shape[0])
+            ineqs = ['lin_ineq', 'lb', 'ub'] if checkBoxBounds else ['lin_ineq']
             eqs = ['lin_eq']
             if self.p._baseClassName == 'NonLin':
                 ineqs.append('c')
                 eqs.append('h')
-            elif self.p.probType in ['MILP', 'MINLP']:
-                pass
+#            elif self.p.probType in ['MILP', 'MINLP']:
+#                pass
                 #ineqs.append('intConstraints')
             for field in ineqs:
-                fv = array(getattr(self, field)()).flatten()
+                fv = array(getattr(self, field)())#.flatten()
                 if fv.size > 0:
-                    #ind_max = argmax(fv)
-                    #val_max = fv[ind_max
-                    val_max = nanmax(fv)
-                    if not isnan(val_max):
+                    val_max = nanmax(fv, fv.ndim-1)
+                    r = where(r < val_max, val_max, r) # handles nan ok
+                    if not self.isMultiArray and not isnan(val_max):
                         ind_max = where(fv==val_max)[0][0]
                         if r < val_max:
                             r, ind, fname = val_max, ind_max, field
             for field in eqs:
-                fv = array(getattr(self, field)()).flatten()
+                fv = array(getattr(self, field)())#.flatten()
                 if fv.size > 0:
                     fv = abs(fv)
-                    ind_max = argmax(fv)
-                    val_max = fv[ind_max]
-                    if r < val_max:
-                        r, ind, fname = val_max, ind_max, field
-            self._mr, self._mrName,  self._mrInd= r, fname, ind
+                    val_max = nanmax(fv, fv.ndim-1)
+                    r = where(r < val_max, val_max, r) # handles nan ok
+                    if not self.isMultiArray and not isnan(val_max):
+                        ind_max = argmax(fv)
+                        #val_max = fv[ind_max]
+                        if r < val_max:
+                            r, ind, fname = val_max, ind_max, field
+            if not self.isMultiArray:
+                self._mr, self._mrName,  self._mrInd= r, fname, ind
         if retAll:
             return asscalar(copy(self._mr)), self._mrName, asscalar(copy(self._mrInd))
-        else: return asscalar(copy(self._mr))
+        else: return asscalar(copy(self._mr)) if not self.isMultiArray else r
 
     def sum_of_all_active_constraints(self):
         if not hasattr(self, '_sum_of_all_active_constraints'):
@@ -182,7 +202,7 @@ class Point:
                     
                     # not "+="!!!!! Else some problems with array shapes can occur
                     #print self.f()-bestFeasiblePoint.f()
-#                    val = val + max((0, self.f()-bestFeasiblePoint.f())) * p.contol / p.Ftol 
+#                    val = val + max((0, self.f()-bestFeasiblePoint.f())) * p.contol / p.fTol 
                     pass
 
                 self._mr_alt, self._mrName_alt,  self._mrInd_alt = val, 'all active', 0
@@ -267,26 +287,25 @@ class Point:
         """
         if self.p.isUC:
             return self.f() < point2compare.f()
-            
-        contol = self.p.contol
 
-        
+        #contol = self.p.contol
+
         if altLinInEq:
             mr, point2compareResidual = self.mr_alt(bestFeasiblePoint=bestFeasiblePoint), point2compare.mr_alt(bestFeasiblePoint=bestFeasiblePoint)
         else:
             mr, point2compareResidual =  self.mr(), point2compare.mr()
         
 #        if altLinInEq and bestFeasiblePoint is not None and isfinite(self.f()) and isfinite(point2compare.f()):
-#            Ftol = self.p.Ftol
-#            mr += (self.f()  - bestFeasiblePoint.f() + Ftol) *contol / Ftol
-#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()+Ftol) *contol / Ftol
-#            mr += max((0, self.f()  - bestFeasiblePoint.f())) *contol/ Ftol
-#            point2compareResidual += max((0, point2compare.f() - bestFeasiblePoint.f())) *contol/ Ftol
+#            fTol = self.p.fTol
+#            mr += (self.f()  - bestFeasiblePoint.f() + fTol) *contol / fTol
+#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()+fTol) *contol / fTol
+#            mr += max((0, self.f()  - bestFeasiblePoint.f())) *contol/ fTol
+#            point2compareResidual += max((0, point2compare.f() - bestFeasiblePoint.f())) *contol/ fTol
 #            assert self.f() >= bestFeasiblePoint.f()
 #            assert point2compare.f() >= bestFeasiblePoint.f()
-#            mr += (self.f()  - bestFeasiblePoint.f()) / Ftol
-#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()) / Ftol
-        criticalResidualValue = max((contol, point2compareResidual))
+#            mr += (self.f()  - bestFeasiblePoint.f()) / fTol
+#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()) / fTol
+        #criticalResidualValue = max((contol, point2compareResidual))
         self_nNaNs, point2compare_nNaNs = self.nNaNs(), point2compare.nNaNs()
 
         if point2compare_nNaNs  > self_nNaNs: return True
@@ -298,12 +317,17 @@ class Point:
             elif point2compareResidual > self.p.contol and point2compareResidual > mr: return True
         else: # got here means self_nNaNs = point2compare_nNaNs but not equal to 0
             if mr == 0 and point2compareResidual == 0: 
-                self.p.err('you should provide at least one active constraint in each point from R^n where some constraints are undefined')
+                if self.p.solver.__name__ not in ('interalg', 'de'):
+                    self.p.err('you should provide at least one active constraint in each point from R^n where some constraints are undefined')
             return mr < point2compareResidual
 
         point2compareF_is_NaN = isnan(point2compare.f())
         selfF_is_NaN = isnan(self.f())
-
+        
+        if isPyPy:
+            if type(point2compareF_is_NaN) == ndarray: point2compareF_is_NaN = asscalar(point2compareF_is_NaN)
+            if type(selfF_is_NaN) == ndarray: selfF_is_NaN = asscalar(selfF_is_NaN)
+        
         if not point2compareF_is_NaN: # f(point2compare) is not NaN
             if not selfF_is_NaN: # f(newPoint) is not NaN
                 return self.f() < point2compare.f()
@@ -338,17 +362,21 @@ class Point:
 
     def nNaNs(self):
         # returns number of nans in constraints
-        if self.p._baseClassName != 'NonLin': return 0
-        r = 0
-        c, h = self.c(), self.h()
-        r += len(where(isnan(c))[0])
-        r += len(where(isnan(h))[0])
+        if self.p._baseClassName != 'NonLin': 
+            return 0
+        hasC, hasH = self.p.userProvided.c, self.p.userProvided.h
+        if hasH and not hasattr(self, '_nNaNs_H'):
+            self.h()
+        if hasC and not hasattr(self, '_nNaNs_C'):
+            self.c()
+        r = (self._nNaNs_C if hasC else 0) + (self._nNaNs_H if hasH else 0)
+        #r = len(where(isnan(c))[0])
         return r
     
     def linePoint(self, alp, point2, ls=None):
         # returns alp * point1 + (1-alp) * point2
         # where point1 is self, alp is real number
-        assert isscalar(alp)
+        #assert isscalar(alp)
         p = self.p
         r = p.point(self.x * (1-alp) + point2.x * alp)
         
@@ -368,14 +396,26 @@ class Point:
             c1, c2 = self.c(), point2.c()
             ind1 = logical_or(c1 > 0,  isnan(c1))
             ind2 = logical_or(c2 > 0,  isnan(c2))
-            ind = where(ind1 | ind2)[0]
+            
+            # prev
+#            ind = where(ind1 | ind2)[0]
+#            
+#            _c = zeros(p.nc)
+#            if ind.size != 0:
+#                _c[ind] = p.c(r.x, ind)
+
+            # new, temporary walkaround for PyPy
+            ind = logical_or(ind1, ind2)
             
             _c = zeros(p.nc)
-            if ind.size != 0:
-                _c[ind] = p.c(r.x, ind)
+            if any(ind):
+                _c[ind] = p.c(r.x, where(ind)[0])
             r._c = _c
+            r._nNaNs_C = isnan(_c).sum(_c.ndim-1)
         
         # TODO: mb same for h?
+        
+        #r._linePointDescriptor = alp # (self, alp, point2)
         return r
 
 
@@ -388,35 +428,35 @@ class Point:
 #            if all(isfinite(self.f())): threshold = self.p.contol
 #            else: 0.0 = 0
             lin_eq = self.lin_eq()
-            ind_lb, ind_ub = where(lb>0.0)[0], where(ub>0.0)[0]
-            ind_lin_ineq = where(lin_ineq>0.0)[0]
-            ind_lin_eq = where(abs(lin_eq)>0.0)[0]
+            ind_lb, ind_ub = lb>0.0, ub>0.0
+            ind_lin_ineq = lin_ineq>0.0
+            ind_lin_eq = abs(lin_eq)>0.0
             USE_SQUARES = 1
             if USE_SQUARES:
-                if ind_lb.size != 0:
+                if any(ind_lb):
                     r += sum(lb[ind_lb] ** 2)
-                if ind_ub.size != 0:
+                if any(ind_ub):
                     r += sum(ub[ind_ub] ** 2)
 
 #                if ind_lb.size != 0:
 #                    r += sum(lb[ind_lb])
 #                if ind_ub.size != 0:
 #                    r += sum(ub[ind_ub])
-                if ind_lin_ineq.size != 0:
+                if any(ind_lin_ineq):
                     r += sum(lin_ineq[ind_lin_ineq] ** 2)
-                if ind_lin_eq.size != 0:
+                if any(ind_lin_eq):
                     r += sum(lin_eq[ind_lin_eq] ** 2)
 
                 self._all_lin = r / self.p.contol
 #                self._all_lin = r
             else:
-                if ind_lb.size != 0:
+                if any(ind_lb):
                     r += sum(lb[ind_lb])
-                if ind_ub.size != 0:
+                if any(ind_ub):
                     r += sum(ub[ind_ub])
-                if ind_lin_ineq.size != 0:
+                if any(ind_lin_ineq):
                     r += sum(lin_ineq[ind_lin_ineq])
-                if ind_lin_eq.size != 0:
+                if any(ind_lin_eq):
                     r += sum(abs(lin_eq[ind_lin_eq]))
                 self._all_lin = r
                     
@@ -432,22 +472,23 @@ class Point:
             lb, ub = self.lb(), self.ub()
             lin_ineq = self.lin_ineq()
             lin_eq = self.lin_eq()
-            ind_lb, ind_ub = where(lb > 0.0)[0], where(ub > 0.0)[0]
-            ind_lin_ineq = where(lin_ineq > 0.0)[0]
-            ind_lin_eq = where(abs(lin_eq) != 0.0)[0]
+            ind_lb, ind_ub = lb > 0.0, ub > 0.0
+            ind_lin_ineq = lin_ineq > 0.0
+            ind_lin_eq = abs(lin_eq) != 0.0
             
 
             USE_SQUARES = 1
             if USE_SQUARES:
-                if ind_lb.size != 0:
+                if any(ind_lb):
                     d[ind_lb] -= lb[ind_lb]# d/dx((x-lb)^2) for violated constraints
-                if ind_ub.size != 0:
+                if any(ind_ub):
                     d[ind_ub] += ub[ind_ub]# d/dx((x-ub)^2) for violated constraints
-                if ind_lin_ineq.size != 0:
+                if any(ind_lin_ineq):
                     # d/dx((Ax-b)^2)
-                    b = p.b[ind_lin_ineq]
+                    Ind_lin_ineq = where(ind_lin_ineq)[0]
+                    b = p.b[Ind_lin_ineq]
                     if hasattr(p, '_A'):
-                        a = p._A[ind_lin_ineq] 
+                        a = p._A[Ind_lin_ineq] 
                         tmp = a._mul_sparse_matrix(csr_matrix((self.x, (arange(n), zeros(n))), shape=(n, 1))).toarray().flatten() - b 
                         
                         #tmp = a._mul_sparse_matrix(csr_matrix((self.x, reshape(p.n, 1))).toarray().flatten() - b 
@@ -456,12 +497,13 @@ class Point:
                     else:
                         a = p.A[ind_lin_ineq] 
                         d += dot(a.T, dot(a, self.x)  - b) # d/dx((Ax-b)^2)
-                if ind_lin_eq.size != 0:
+                if any(ind_lin_eq):
+                    #Ind_lin_eq = where(ind_lin_eq)[0]
                     if isspmatrix(p.Aeq):
                         p.err('this solver is not ajusted to handle sparse Aeq matrices yet')
                     #self.p.err('nonzero threshold is not ajusted with lin eq yet')
-                    aeq = p.Aeq#[ind_lin_eq]
-                    beq = p.beq#[ind_lin_eq]
+                    aeq = p.Aeq#[Ind_lin_eq]
+                    beq = p.beq#[Ind_lin_eq]
                     d += dot(aeq.T, dot(aeq, self.x)  - beq) # d/dx((Aeq x - beq)^2)
                     
 
@@ -469,18 +511,19 @@ class Point:
                 self._all_lin_gradient = 2.0 * d / p.contol
 
             else:
-                if ind_lb.size != 0:
+                assert 0
+                if any(ind_lb):
                     d[ind_lb] -= 1# d/dx(lb-x) for violated constraints
-                if ind_ub.size != 0:
+                if any(ind_ub):
                     d[ind_ub] += 1# d/dx(x-ub) for violated constraints
-                if ind_lin_ineq.size != 0:
+                if any(ind_lin_ineq):
                     # d/dx(Ax-b)
                     b = p.b[ind_lin_ineq]
                     if hasattr(p, '_A'):
                         d += (p._A[ind_lin_ineq]).sum(0).A.flatten()
                     else:
                         d += (p.A[ind_lin_ineq]).sum(0).flatten()
-                if ind_lin_eq.size != 0:
+                if any(ind_lin_eq):
                     # currently for ralg it should be handled in dilation matrix
                     p.err('not implemented yet, if you see it inform OpenOpt developers')
 #                    beq = p.beq[ind_lin_eq]
@@ -515,8 +558,9 @@ class Point:
                 th = 0.0
                 #th = contol / 2.0
                 C = p.c(x)
-                ind = where(C>th)[0]
-                activeC = C[ind]
+                Ind = C>th
+                ind = where(Ind)[0]
+                activeC = asarray(C[Ind])# asarray and Ind for PyPy compatibility
                 if len(ind) > 0:
                     tmp = p.dc(x, ind)
 
@@ -534,15 +578,16 @@ class Point:
                             
                     if tmp.ndim > 1:
                         tmp = tmp.sum(0)
-                    direction += (tmp.A if isspmatrix(tmp) or isinstance(tmp, matrix) else tmp).flatten()
+                    direction += (tmp.A if type(tmp) != ndarray else tmp).flatten()
             
 
             if p.userProvided.h:
                 #th = 0.0
                 th = contol / 2.0
                 H = p.h(x)
-                ind1 = where(H>th)[0]
-                H1 = H[ind1]
+                Ind1 = H>th
+                ind1 = where(Ind1)[0]
+                H1 = asarray(H[Ind1])# asarray and Ind1 for PyPy compatibility
                 if len(ind1) > 0:
                     tmp = p.dh(x, ind1)
                     
@@ -560,7 +605,7 @@ class Point:
                     
                     if tmp.ndim > 1: 
                         tmp = tmp.sum(0)
-                    direction += (tmp.A if isspmatrix(tmp) or isinstance(tmp, matrix) else tmp).flatten()
+                    direction += (tmp.A if isspmatrix(tmp) or hasattr(tmp, 'toarray') else tmp).flatten()
                 ind2 = where(H<-th)[0]
                 H2 = H[ind2]
                 if len(ind2) > 0:
@@ -579,7 +624,7 @@ class Point:
                     
                     if tmp.ndim > 1: 
                         tmp = tmp.sum(0)
-                    direction -= (tmp.A if isspmatrix(tmp) or isinstance(tmp, matrix) else tmp).flatten()
+                    direction -= (tmp.A if type(tmp) != ndarray else tmp).flatten()
             self._sum_of_all_active_constraints_gradient = direction
         return Copy(self._sum_of_all_active_constraints_gradient)
 
@@ -611,20 +656,20 @@ class Point:
                     #if p.debug: assert array_equal(self.dh(ind).flatten(), self.dmr())
                     self.dType = 'h'
                 else:
-                    p.err('error in getRalgDirection (unknown residual type ' + fname + ' ), you should report the bug')
+                    self.p.err('error in getDirection (unknown residual type ' + fname + ' ), you should report the bug')
                 self.direction = d.flatten()
 
             if type(self.direction) != ndarray: self.direction = self.direction.A.flatten()
             
 #            print 'currBestFeasPoint is not None:', (currBestFeasPoint is not None), 'self.f() > currBestFeasPoint.f():', self.f() > currBestFeasPoint.f()
 
-            contol, Ftol = self.p.contol, self.p.Ftol
-#            print self.f(),  currBestFeasPoint.f(), self.f() - Ftol ==  currBestFeasPoint.f()
+            contol, fTol = self.p.contol, self.p.fTol
+#            print self.f(),  currBestFeasPoint.f(), self.f() - fTol ==  currBestFeasPoint.f()
             
-            if currBestFeasPoint is not None and self.f() + 0.25*Ftol > currBestFeasPoint.f():
+            if currBestFeasPoint is not None and self.f() + 0.25*fTol > currBestFeasPoint.f():
                 
-                #self.direction += ((self.f()-currBestFeasPoint.f())  * p.contol / nDF / Ftol) * DF
-#                self.direction += self.df() * (contol/Ftol)       
+                #self.direction += ((self.f()-currBestFeasPoint.f())  * p.contol / nDF / fTol) * DF
+#                self.direction += self.df() * (contol/fTol)       
                 pass
                     
             return self.direction.copy() # it may be modified in ralg when some constraints coords are NaN
